"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProjectBuilder = exports.ProjectBuilder = void 0;
const tslib_1 = require("tslib");
const SchemaParser_1 = require("../parser/SchemaParser");
const resultHelper_1 = require("../utils/resultHelper");
const ModuleBuilder_1 = require("./ModuleBuilder");
const error_1 = require("../types/error");
const const_1 = require("../const");
class ProjectBuilder {
    constructor(builderOptions) {
        let customBuilderOptions = builderOptions;
        if (typeof builderOptions.customizeBuilderOptions === 'function') {
            customBuilderOptions =
                builderOptions.customizeBuilderOptions(builderOptions);
        }
        const { template, plugins, postProcessors, schemaParser = new SchemaParser_1.SchemaParser(), projectPreProcessors = [], projectPostProcessors = [], inStrictMode = false, extraContextData = {}, } = customBuilderOptions;
        this.template = template;
        this.plugins = plugins;
        this.postProcessors = postProcessors;
        this.schemaParser = schemaParser;
        this.projectPreProcessors = projectPreProcessors;
        this.projectPostProcessors = projectPostProcessors;
        this.inStrictMode = inStrictMode;
        this.extraContextData = extraContextData;
    }
    generateProject(originalSchema) {
        var _a, _b, _c;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // Init
            const { schemaParser } = this;
            let schema = typeof originalSchema === 'string'
                ? JSON.parse(originalSchema)
                : originalSchema;
            // Parse / Format
            // Preprocess 前置处理器
            for (const preProcessor of this.projectPreProcessors) {
                // eslint-disable-next-line no-await-in-loop
                schema = yield preProcessor(schema);
            }
            // Validate
            if (!schemaParser.validate(schema)) {
                throw new error_1.CodeGeneratorError('Schema is invalid');
            }
            // Collect Deps
            // Parse JSExpression
            const parseResult = schemaParser.parse(schema);
            const projectRoot = yield this.template.generateTemplate(parseResult);
            let buildResult = [];
            const builders = this.createModuleBuilders({
                extraContextData: {
                    projectRemark: (_a = parseResult === null || parseResult === void 0 ? void 0 : parseResult.project) === null || _a === void 0 ? void 0 : _a.projectRemark,
                    template: this.template,
                },
            });
            // Generator Code module
            // components
            // pages
            const containerBuildResult = yield Promise.all(parseResult.containers.map((containerInfo) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let builder;
                let path;
                if (containerInfo.containerType === 'Page') {
                    builder = builders.pages;
                    path = this.template.slots.pages.path;
                }
                else {
                    builder = builders.components;
                    path = this.template.slots.components.path;
                }
                const { files } = yield builder.generateModule(containerInfo);
                return {
                    moduleName: containerInfo.moduleName,
                    path,
                    files,
                };
            })));
            buildResult = buildResult.concat(containerBuildResult);
            // router
            if (parseResult.globalRouter && builders.router) {
                const { files } = yield builders.router.generateModule(parseResult.globalRouter);
                buildResult.push({
                    path: this.template.slots.router.path,
                    files,
                });
            }
            // entry
            if (parseResult.project && builders.entry) {
                const { files } = yield builders.entry.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.entry.path,
                    files,
                });
            }
            // appConfig
            if (builders.appConfig) {
                const { files } = yield builders.appConfig.generateModule(parseResult);
                buildResult.push({
                    path: this.template.slots.appConfig.path,
                    files,
                });
            }
            // buildConfig
            if (builders.buildConfig) {
                const { files } = yield builders.buildConfig.generateModule(parseResult);
                buildResult.push({
                    path: this.template.slots.buildConfig.path,
                    files,
                });
            }
            // constants?
            if (parseResult.project &&
                builders.constants &&
                this.template.slots.constants) {
                const { files } = yield builders.constants.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.constants.path,
                    files,
                });
            }
            // utils?
            if (parseResult.globalUtils &&
                builders.utils &&
                this.template.slots.utils) {
                const { files } = yield builders.utils.generateModule(parseResult.globalUtils);
                buildResult.push({
                    path: this.template.slots.utils.path,
                    files,
                });
            }
            // i18n?
            if (builders.i18n && this.template.slots.i18n) {
                const { files } = yield builders.i18n.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.i18n.path,
                    files,
                });
            }
            // globalStyle
            if (parseResult.project && builders.globalStyle) {
                const { files } = yield builders.globalStyle.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.globalStyle.path,
                    files,
                });
            }
            // htmlEntry
            if (parseResult.project && builders.htmlEntry) {
                const { files } = yield builders.htmlEntry.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.htmlEntry.path,
                    files,
                });
            }
            // packageJSON
            if (parseResult.project && builders.packageJSON) {
                const { files } = yield builders.packageJSON.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.packageJSON.path,
                    files,
                });
            }
            // demo
            if (parseResult.project && builders.demo) {
                const { files } = yield builders.demo.generateModule(parseResult.project);
                buildResult.push({
                    path: this.template.slots.demo.path,
                    files,
                });
            }
            // handle extra slots
            yield this.generateExtraSlots(builders, parseResult, buildResult);
            // Post Process
            const isSingleComponent = (_c = (_b = parseResult === null || parseResult === void 0 ? void 0 : parseResult.project) === null || _b === void 0 ? void 0 : _b.projectRemark) === null || _c === void 0 ? void 0 : _c.isSingleComponent;
            // Combine Modules
            buildResult.forEach((moduleInfo) => {
                let targetDir = getDirFromRoot(projectRoot, moduleInfo.path);
                // if project only contain single component, skip creation of directory.
                if (moduleInfo.moduleName && !isSingleComponent) {
                    const dir = (0, resultHelper_1.createResultDir)(moduleInfo.moduleName);
                    (0, resultHelper_1.addDirectory)(targetDir, dir);
                    targetDir = dir;
                }
                moduleInfo.files.forEach((file) => (0, resultHelper_1.addFile)(targetDir, file));
            });
            // post-processors
            let finalResult = projectRoot;
            for (const projectPostProcessor of this.projectPostProcessors) {
                // eslint-disable-next-line no-await-in-loop
                finalResult = yield projectPostProcessor(finalResult, schema, originalSchema, {
                    template: this.template,
                    parseResult,
                });
            }
            return finalResult;
        });
    }
    createModuleBuilders(extraContextData = {}) {
        const builders = {};
        Object.keys(this.plugins).forEach((pluginName) => {
            if (this.plugins[pluginName].length > 0) {
                const options = {};
                if (this.template.slots[pluginName] &&
                    this.template.slots[pluginName].fileName) {
                    options.mainFileName = this.template.slots[pluginName].fileName;
                }
                builders[pluginName] = (0, ModuleBuilder_1.createModuleBuilder)(Object.assign({ plugins: this.plugins[pluginName], postProcessors: this.postProcessors, contextData: Object.assign(Object.assign({ 
                        // template: this.template,
                        inStrictMode: this.inStrictMode, tolerateEvalErrors: true, evalErrorsHandler: '' }, this.extraContextData), extraContextData) }, options));
                console.log(builders[pluginName]);
            }
        });
        return builders;
    }
    generateExtraSlots(builders, parseResult, buildResult) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const slotName in this.template.slots) {
                if (!(0, const_1.isBuiltinSlotName)(slotName)) {
                    const { files } = yield builders[slotName].generateModule(parseResult);
                    buildResult.push({
                        path: this.template.slots[slotName].path,
                        files,
                    });
                }
            }
        });
    }
}
exports.ProjectBuilder = ProjectBuilder;
function createProjectBuilder(initOptions) {
    return new ProjectBuilder(initOptions);
}
exports.createProjectBuilder = createProjectBuilder;
function getDirFromRoot(root, path) {
    let current = root;
    path.forEach((p) => {
        const exist = current.dirs.find((d) => d.name === p);
        if (exist) {
            current = exist;
        }
        else {
            const newDir = (0, resultHelper_1.createResultDir)(p);
            (0, resultHelper_1.addDirectory)(current, newDir);
            current = newDir;
        }
    });
    return current;
}
//# sourceMappingURL=ProjectBuilder.js.map