"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeDirsFromResult = exports.removeFilesFromResult = exports.findDir = exports.globDirs = exports.scanDirs = exports.getFileNameWithExt = exports.scanFiles = exports.globFiles = exports.findFile = exports.flattenResult = exports.addFile = exports.addDirectory = exports.createResultDir = exports.createResultFile = void 0;
const tslib_1 = require("tslib");
const nanomatch_1 = tslib_1.__importDefault(require("nanomatch"));
const error_1 = require("../types/error");
function createResultFile(name, ext = 'jsx', content = '') {
    return {
        name,
        ext,
        content,
    };
}
exports.createResultFile = createResultFile;
function createResultDir(name) {
    return {
        name,
        dirs: [],
        files: [],
    };
}
exports.createResultDir = createResultDir;
function addDirectory(target, dir) {
    if (target.dirs.findIndex((d) => d.name === dir.name) < 0) {
        target.dirs.push(dir);
    }
    else {
        throw new error_1.CodeGeneratorError(`Adding same directory to one directory: ${dir.name} -> ${target.name}`);
    }
}
exports.addDirectory = addDirectory;
function addFile(target, file) {
    if (target.files.findIndex((f) => f.name === file.name && f.ext === file.ext) < 0) {
        target.files.push(file);
    }
    else {
        throw new error_1.CodeGeneratorError(`Adding same file to one directory: ${file.name} -> ${target.name}`);
    }
}
exports.addFile = addFile;
function flattenResult(dir, cwd = '') {
    if (!dir.files.length && !dir.dirs.length) {
        return [];
    }
    return [
        ...dir.files.map((file) => ({
            pathName: joinPath(cwd, `${file.name}${file.ext ? `.${file.ext}` : ''}`),
            content: file.content,
        })),
    ].concat(...dir.dirs.map((subDir) => flattenResult(subDir, joinPath(cwd, subDir.name))));
}
exports.flattenResult = flattenResult;
/**
 * 查找文件
 * @param result 出码结果
 * @param fileGlobExpr 文件名匹配表达式
 * @param resultDirPath 出码结果的路径（默认是 '.'）
 * @returns 匹配的第一个文件或 null （找不到）
 */
function findFile(result, fileGlobExpr, options = {}, resultDirPath = getResultNameOrDefault(result, '')) {
    const maxDepth = !/\/|\*\*/.test(fileGlobExpr) ? 1 : undefined; // 如果 glob 表达式里面压根不会匹配子目录，则深度限制为 1
    const files = scanFiles(result, resultDirPath, maxDepth);
    for (let [filePath, file] of files) {
        if (nanomatch_1.default.isMatch(filePath, fileGlobExpr, options)) {
            return file;
        }
    }
    return null;
}
exports.findFile = findFile;
/**
 * 使用 glob 语法查找多个文件
 * @param result 出码结果
 * @param fileGlobExpr 文件名匹配表达式
 * @param resultDirPath 出码结果的路径（默认是 '.'）
 * @returns 找到的文件列表的迭代器 [ [文件路径, 文件信息], ... ]
 */
function* globFiles(result, fileGlobExpr, options = {}, resultDirPath = getResultNameOrDefault(result, '')) {
    const files = scanFiles(result, resultDirPath);
    for (let [filePath, file] of files) {
        if (nanomatch_1.default.isMatch(filePath, fileGlobExpr, options)) {
            yield [filePath, file];
        }
    }
}
exports.globFiles = globFiles;
/**
 * 遍历所有的文件
 */
function* scanFiles(result, resultDirPath = getResultNameOrDefault(result, ''), maxDepth = 10000) {
    for (let file of result.files) {
        const fileName = getFileNameWithExt(file);
        yield [joinPath(resultDirPath, fileName), file];
    }
    for (let subDir of result.dirs) {
        yield* scanFiles(subDir, joinPath(resultDirPath, subDir.name), maxDepth - 1);
    }
}
exports.scanFiles = scanFiles;
function getFileNameWithExt(file) {
    return `${file.name}${file.ext ? `.${file.ext}` : ''}`;
}
exports.getFileNameWithExt = getFileNameWithExt;
function getResultNameOrDefault(result, defaultDir = '/') {
    return result.name && result.name !== '.' ? result.name : defaultDir;
}
function joinPath(...pathParts) {
    return pathParts
        .filter((x) => x !== '' && x !== '.')
        .join('/')
        .replace(/\\+/g, '/')
        .replace(/\/+/g, '/');
}
function* scanDirs(result, resultDirPath = getResultNameOrDefault(result, ''), maxDepth = 10000) {
    yield [resultDirPath, result];
    for (let subDir of result.dirs) {
        yield* scanDirs(subDir, joinPath(resultDirPath, subDir.name), maxDepth - 1);
    }
}
exports.scanDirs = scanDirs;
function* globDirs(result, dirGlobExpr, options = {}, resultDirPath = getResultNameOrDefault(result, '')) {
    const dirs = scanDirs(result, resultDirPath);
    for (let [dirPath, dir] of dirs) {
        if (nanomatch_1.default.isMatch(dirPath, dirGlobExpr, options)) {
            yield [dirPath, dir];
        }
    }
}
exports.globDirs = globDirs;
function findDir(result, dirGlobExpr, options = {}, resultDirPath = getResultNameOrDefault(result, '')) {
    const dirs = scanDirs(result, resultDirPath);
    for (let [dirPath, dir] of dirs) {
        if (nanomatch_1.default.isMatch(dirPath, dirGlobExpr, options)) {
            return dir;
        }
    }
    return null;
}
exports.findDir = findDir;
/**
 * 从结果中移除一些文件
 * @param result 出码结果目录
 * @param filePathGlobExpr 要移除的文件路径（glob 表达式）
 * @param globOptions glob 参数
 * @returns 移除了多少文件
 */
function removeFilesFromResult(result, filePathGlobExpr, globOptions = {}) {
    let removedCount = 0;
    const [dirPath, fileName] = splitPath(filePathGlobExpr);
    const dirs = dirPath ? globDirs(result, dirPath) : [['', result]];
    for (let [, dir] of dirs) {
        const files = globFiles(dir, fileName, globOptions, '.');
        for (let [, file] of files) {
            dir.files.splice(dir.files.indexOf(file), 1);
            removedCount += 1;
        }
    }
    return removedCount;
}
exports.removeFilesFromResult = removeFilesFromResult;
/**
 * 从结果中移除一些目录
 * @param result 出码结果目录
 * @param dirPathGlobExpr 要移除的目录路径（glob 表达式）
 * @param globOptions glob 参数
 * @returns 移除了多少文件
 */
function removeDirsFromResult(result, dirPathGlobExpr, globOptions = {}) {
    let removedCount = 0;
    const [dirPath, fileName] = splitPath(dirPathGlobExpr);
    const dirs = dirPath ? globDirs(result, dirPath) : [['', result]];
    for (let [, dir] of dirs) {
        const foundDirs = globDirs(dir, fileName, globOptions, '.');
        for (let [, foundDir] of foundDirs) {
            dir.dirs.splice(dir.dirs.indexOf(foundDir), 1);
            removedCount += 1;
        }
    }
    return removedCount;
}
exports.removeDirsFromResult = removeDirsFromResult;
/**
 * 将文件路径拆分为目录路径和文件名
 * @param filePath
 * @returns [fileDirPath, fileName]
 */
function splitPath(filePath) {
    const parts = filePath.split('/');
    const fileName = parts.pop() || '';
    return [joinPath(...parts), fileName];
}
//# sourceMappingURL=resultHelper.js.map