import changeCase from 'change-case';
import {
  COMMON_CHUNK_NAME,
  CLASS_DEFINE_CHUNK_NAME,
  FC_DEFINE_CHUNK_NAME,
  FUN_DEFINE_CHUNK_NAME,
  DEFAULT_LINK_AFTER,
} from '../../../const/generator';
import { REACT_CHUNK_NAME } from './const';

import {
  BuilderComponentPlugin,
  BuilderComponentPluginFactory,
  ChunkType,
  FileType,
  ICodeStruct,
  IContainerInfo,
} from '../../../types';
import { ensureValidClassName } from '../../../utils/validate';

const pluginFactory: BuilderComponentPluginFactory<unknown> = () => {
  const plugin: BuilderComponentPlugin = async (pre: ICodeStruct) => {
    const next: ICodeStruct = {
      ...pre,
    };

    const ir = next.ir as IContainerInfo;

    // 将模块名转换成 PascalCase 的格式，并添加特定后缀，防止命名冲突
    const componentClassName = ensureValidClassName(
      `${changeCase.pascalCase(ir.moduleName)}$$${ir.containerType}`
    );

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.Start,
      content: `const ${componentClassName} = (props) => {`,
      linkAfter: [
        COMMON_CHUNK_NAME.ExternalDepsImport,
        COMMON_CHUNK_NAME.InternalDepsImport,
        COMMON_CHUNK_NAME.ImportAliasDefine,
        COMMON_CHUNK_NAME.FileVarDefine,
        COMMON_CHUNK_NAME.FileUtilDefine,
      ],
    });

    if (ir.containerType === 'Component') {
      next.chunks.push({
        type: ChunkType.STRING,
        fileType: FileType.TSX,
        name: FC_DEFINE_CHUNK_NAME.InsVar,
        content: `${componentClassName}.displayName = '${componentClassName}';`,
        linkAfter: [FC_DEFINE_CHUNK_NAME.Start],
      });
    }

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.End,
      content: '}',
      linkAfter: [
        ...DEFAULT_LINK_AFTER[FUN_DEFINE_CHUNK_NAME.End],
        // REACT_CHUNK_NAME.FcRenderEnd,
        // REACT_CHUNK_NAME.FcDidMountEnd,
      ],
    });

    // 构造器暂时不需要？

    /* next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FC_DEFINE_CHUNK_NAME.ConstructorStart,
      content: 'constructor(props, context) { super(props); ',
      linkAfter: [...DEFAULT_LINK_AFTER[FC_DEFINE_CHUNK_NAME.ConstructorStart]],
    });

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FC_DEFINE_CHUNK_NAME.ConstructorEnd,
      content: '}',
      linkAfter: [...DEFAULT_LINK_AFTER[FC_DEFINE_CHUNK_NAME.ConstructorEnd]],
    }); */

    // React中的一个hook函数

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.EffectHookStart,
      content: 'useEffect(() => {',
      linkAfter: [...DEFAULT_LINK_AFTER[FUN_DEFINE_CHUNK_NAME.EffectHookStart]],
    });

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.EffectHookEnd,
      content: ' }, []);',
      linkAfter: [
        ...DEFAULT_LINK_AFTER[FUN_DEFINE_CHUNK_NAME.EffectHookEnd],
        // REACT_CHUNK_NAME.FcDidMountContent,
        // REACT_CHUNK_NAME.FcDidMountStart,
      ],
    });

    // 替换render

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.ReturnStart,
      content: 'return (',
      linkAfter: [
        ...DEFAULT_LINK_AFTER[FUN_DEFINE_CHUNK_NAME.ReturnStart],
        // REACT_CHUNK_NAME.FcDidMountEnd,
      ],
    });

    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: FUN_DEFINE_CHUNK_NAME.ReturnEnd,
      content: ');',
      linkAfter: [
        ...DEFAULT_LINK_AFTER[FUN_DEFINE_CHUNK_NAME.ReturnEnd],
        // REACT_CHUNK_NAME.FcRenderStart,
        // REACT_CHUNK_NAME.FcRenderPre,
        // REACT_CHUNK_NAME.FcRenderJSX,
      ],
    });

    // 输出 export
    next.chunks.push({
      type: ChunkType.STRING,
      fileType: FileType.TSX,
      name: COMMON_CHUNK_NAME.FileExport,
      content: `export default ${componentClassName};`,
      linkAfter: [
        COMMON_CHUNK_NAME.ExternalDepsImport,
        COMMON_CHUNK_NAME.InternalDepsImport,
        COMMON_CHUNK_NAME.ImportAliasDefine,
        COMMON_CHUNK_NAME.FileVarDefine,
        COMMON_CHUNK_NAME.FileUtilDefine,
        // FC_DEFINE_CHUNK_NAME.End,
        FUN_DEFINE_CHUNK_NAME.End,
      ],
    });

    return next;
  };
  return plugin;
};

export default pluginFactory;
